% *************************************************************
% author:   Gene Cheung                                      **
% date:     07/23/2014                                       **
% modified: 07/25/2014                                       **
% purpose:  convert contours to gradient maps                **
% *************************************************************

function [hGrad, vGrad] = contour2edgeMap (startPt, dcode, imHeight, imWidth)

hGrad = zeros(imHeight, imWidth-1);
vGrad = zeros(imHeight-1, imWidth);
[dHeight, ~] = size(dcode);

% rebuild gradient map 1 edge at a time ********
for i = 1:dHeight,
    
    row = startPt(i).row;      % contour starting point
    col = startPt(i).col;
    if startPt(i).hGrad == 1,
        hGrad(row,col) = 1;
    else
        vGrad(row,col) = 1;
    end
    
    [~, ind] = min(dcode(i,:));
    dcode1 = dcode(i,1:ind-1);  
    lastDirect = dcode1(1);
    
    for j = 2:ind-1,
        
        if lastDirect == 1,  % last direction is north ********
            if dcode1(j) == 1 && row > 1,                        % north
                hGrad(row-1,col) = 1;
                row = row-1;
            elseif dcode1(j) == 2 && row > 1 && col < imWidth,   % east
                vGrad(row-1,col+1) = 1;
                row = row-1;
                col = col+1;
                lastDirect = 2;
            elseif dcode1(j) == 4 && row > 1,                    % west
                vGrad(row-1,col) = 1;
                row = row-1;
                lastDirect = 4;
            end
            
        elseif lastDirect == 2,  % last direction is east ********
            if dcode1(j) == 1,                                   % north
                hGrad(row,col) = 1;
                lastDirect = 1;
            elseif dcode1(j) == 2 && col < imWidth,              % east
                vGrad(row, col+1) = 1;
                col = col+1;
            elseif dcode1(j) == 3 && row < imHeight,             % south
                hGrad(row+1,col) = 1;
                lastDirect = 3;
                row = row+1;
            end
            
        elseif lastDirect == 3,  % last direction is south ********
            if dcode1(j) == 2 && col < imWidth,                  % east
                vGrad(row,col+1) = 1;   
                lastDirect = 2;
                col = col+1;
            elseif dcode1(j) == 3 && row < imHeight,             % south
                hGrad(row+1,col) = 1;
                row = row+1;
            elseif dcode1(j) == 4,                               % west
                vGrad(row,col) = 1;
                lastDirect = 4;
            end
            
        elseif lastDirect == 4,  % last direction is west ********
            if dcode1(j) == 3 && col > 1,                        % south
                hGrad(row+1,col-1) = 1;
                lastDirect = 3;
                row = row+1;
                col = col-1;
            elseif dcode1(j) == 4 && col > 1,                    % west
                vGrad(row,col-1) = 1;
                col = col-1;
            elseif dcode1(j) == 1 && col > 1,                    % north
                hGrad(row,col-1) = 1;
                lastDirect = 1;
                col = col-1;
            end
        end
        
        
    end
        
end