% ************************************************************************
% author:   Amin Zheng                                                  **
% date:     06/23/2015                                                  **
% modified:                                                             **
% purpose:  get the contour and starting points from                   **
%           input image                                                 **
% ************************************************************************

function [startPt, dcode, conThresh] = getContourStartPnt(im1, alpha, minLength)

[imRow, imCol] = size(im1);

% down-sampled frame for faster processing ******
scale = 1.0/1;
if scale ~= 1,
    im1 = imresize(im1, scale);
    [imRow, imCol] = size(im1);   % dimension of actual processed image
end

% step 1: compute horizontal, vertical gradients ******
hGrad = zeros(imRow,imCol-1);  % horizontal gradient
vGrad = zeros(imRow-1,imCol);  % vertical gradients
for row = 1:imRow,
    hGrad(row,:) = abs(im1(row, 1:imCol-1) - im1(row, 2:imCol));
end
for col = 1:imCol,
    vGrad(:,col) = abs(im1(1:imRow-1,col) - im1(2:imRow,col));
end
gMean = mean([hGrad(:)' vGrad(:)']);
gVar = var([hGrad(:)' vGrad(:)']);
conThresh = gMean + alpha*sqrt(gVar);            % mean + 1.0 std
hGrad = hGrad > conThresh;
vGrad = vGrad > conThresh;
%     figure(1);
%     displayEdgeMap(hGrad, vGrad, im1);

% step 2: read out contour for encoding ******
[startPt0, dcode0] = edgeMap2contour(hGrad, vGrad);
[dRow, ~] = size(dcode0);
clear startPt;
dcode = [];
count = 0;
for i = 1:dRow,
    ind = find(dcode0(i,:) == 0);
    if ind(1)-1 >= minLength,   % remove contour with length < minLength
        count = count+1;
        startPt(count).hGrad = startPt0(i).hGrad;
        startPt(count).row = startPt0(i).row;
        startPt(count).col = startPt0(i).col;
        dcode(count, :) = dcode0(i,:);
    end
end

figure(1);
[hGrad2, vGrad2] = contour2edgeMap(startPt, dcode, imRow, imCol);
displayEdgeMap(hGrad2, vGrad2, im1);

end

