% ************************************************************************
% author:   Amin Zheng                                                  **
% date:     08/12/2015                                                  **
% modified:                                                             **
% purpose:  code the starting points                                    **
%                                                                       **
% *************************** Parameters *********************************
% imRow:    image height                                                **
% imCol:    image width                                                 **
% startPt:  starting positions of contours                              **
% dRow:     number of contours                                          **
% ************************************************************************

function [bitsStartPnt, golombK1] = golombCodeStartPnt(imRow, imCol, startPt, dRow)
bitsStartPnt1 = 0;
coordStart = zeros(dRow, 2);

% step 1: sort the starting points according to the longest dimension ***
for i = 1:dRow
    if imCol >= imRow
        coordStart(i, 1) = startPt(i).col;
        coordStart(i, 2) = startPt(i).row;
        longSize = imCol;
        shortSize = imRow;
    else
        coordStart(i, 1) = startPt(i).row;
        coordStart(i, 2) = startPt(i).col;
        longSize = imRow;
        shortSize = imCol;
    end
end

[~, ind] = sort(coordStart(:,1));
coordStart = coordStart(ind,:);

% remark: the decoder knows the image height and image width ***
% Step 2: using golomb code to code the longest dimension first ***

% Step 2.1: coding the first point ***
bitsStartPnt1 = bitsStartPnt1 + ceil(log2(longSize)); % code the first point

% Step 2.2: golomb coding the differences by traversaing all the possible parameters ***
%          We choose M = 2^k to make the case simple, so the quotient requires floor(x/2^k)+1 bits ***
%              and the remainder requires k bits  ***
%          Please refer to: https://en.wikipedia.org/wiki/Golomb_coding ***

maxK1 = ceil(log2(longSize-1)); % longSize-1 is maximum difference
tempTotalBits1 = zeros(maxK1+1, 1);
diff1 = coordStart(2:dRow,1) - coordStart(1:dRow-1,1);

for k = 0:maxK1
    if k == maxK1 % binary code
        tempTotalBits1(k+1) = (dRow - 1) * k;
    else
        for i = 1:dRow - 1
            tempTotalBits1(k+1) = tempTotalBits1(k+1) + floor(diff1(i)/(2^k)) + 1 + k;
        end
    end
end

[minBits1, ind] = min(tempTotalBits1);
golombK1 = ind - 1;
bitsStartPnt1 = bitsStartPnt1 + minBits1; % bits of coding the remaining points

% Step 2.3: coding the parameter (0,1,..., maxK) in total maxK + 1 candidates***
bitsStartPnt1 = bitsStartPnt1 + ceil(log2(maxK1+1));


% Step 3: code the remaining dimension ***

bitsStartPnt2 = ceil(log2(shortSize)) * dRow; % using binary coding

bitsStartPnt = bitsStartPnt1 + bitsStartPnt2;
end