% ************************************************************************
% author:   Amin Zheng                                                  **
% date:     11/25/2015                                                  **
% modified:                                                             **
% purpose:  print the coding infor of different contexts                **
% ************************************************************************


function printContextBits(fOutName, contextInfor, countStateSpace)

[maxOrder, ~, ~] = size(countStateSpace);

contextNum = contextInfor(:, :, 1);
contextBit = contextInfor(:, :, 2);

bitsTh = 0;

bitsOutFid = fopen(fOutName, 'wt');

totalUsedTrainSymbols = 0;
for i = 1:maxOrder
    for j = 1:3^i
        if contextBit(i, j) > bitsTh;
            
            % get the contex
            ctx = zeros(i,1);
            temJ = j;
            for k = 1:i
                ctx(k) = floor((temJ - 1)/ (3^(i-k))) + 1;
                temJ = temJ - (3^(i-k)) * (ctx(k)-1);
            end
            
            % get the maximum distance of the context
            depth = i;
            if depth >= 2
                xd = zeros(depth+1,1);
                yd = zeros(depth+1,1);
                
                orient = 0; % 0,1,2,3 ~ north, east, south, west
                yd(2) = yd(1) + 1;
                
                for indCor = 2:depth,
                    if ctx(indCor) == 1     % left
                        orient = mod(orient-1, 4);
                    elseif ctx(indCor) == 3 % rigth
                        orient = mod(orient+1, 4);
                    end
                    
                    if orient == 0,       % North
                        xd(indCor+1) = xd(indCor);
                        yd(indCor+1) = yd(indCor)+1;
                    elseif orient == 1,   % East
                        xd(indCor+1) = xd(indCor)+1;
                        yd(indCor+1) = yd(indCor);
                    elseif orient == 2,
                        xd(indCor+1) = xd(indCor);  % South
                        yd(indCor+1) = yd(indCor)-1;
                    else                  % West
                        xd(indCor+1) = xd(indCor)-1;
                        yd(indCor+1) = yd(indCor);
                    end
                end
                
                dist = zeros(depth-1, 1);
                for indCor = 2:depth,
                    dist(indCor-1) = abs((xd(indCor)-xd(1))*(yd(depth+1)-yd(1))-...
                        (yd(indCor)-yd(1))*(xd(depth+1)-xd(1)))/...
                        sqrt((xd(depth+1)-xd(1))^2+(yd(depth+1)-yd(1))^2);
                end
                ctxDist = max(dist);
            else
                ctxDist = 0;
            end
            
            % get the number of turning direction of the ctx
            ctxTurn = abs(numel(find(ctx == 1))- numel(find(ctx==3)));
            
            % get the estimated entropy
            
            
            count = countStateSpace(i, j, :);
            deltaOffset = 1/3;
            probEsti = (count + deltaOffset) / (3*deltaOffset + sum(count)); % add offset to avoid zero
            
            ctxEstEntropy = -(probEsti(1)*log(probEsti(1))+probEsti(2)*log(probEsti(2))...
                + probEsti(3)*log(probEsti(3)));
            
            ctxStr =  num2str(ctx, '%d');
            fprintf(bitsOutFid, '%s, %.2f, %d, num:%d, bits:%d, ave:%.2f, estAve:%.2f, estNum:%d \n', ...
                ctxStr, ctxDist, ctxTurn, contextNum(i,j), contextBit(i,j)...
                ,contextBit(i,j)/contextNum(i,j), ctxEstEntropy, sum(count));
            
            totalUsedTrainSymbols = totalUsedTrainSymbols + sum(count);
        end
    end
    
end

tempTotalTrain = countStateSpace(1,:,:);

fprintf(bitsOutFid, '\nTotal test symbols:%d, Total train symbols:%d, Total used train symbols:%d\n', ...
    sum(contextNum(:)), sum(tempTotalTrain(:)), totalUsedTrainSymbols);

fclose (bitsOutFid);

end