% ************************************************************************
% author:   Amin Zheng                                                  **
% date:     11/21/2015                                                  **
% modified:                                                             **
% purpose:  get initial context tree and maximum depth                  **
% ************************************************************************

function [countStateSpace, orderRange, minCount] = probTrainMCT(flag, numImg, imgContSet, treePrune, maxCounters)

% step 1: variables initialization

% first consider maxCountersRedun, then choose maxCounters from maxCountersRedun ***
maxCountersRedun = 2 * maxCounters;

% step 2: put the conts in all the imgs into one matrix ***
rowSet = zeros(numImg, 1); % store cont number of all the imgs
lenSet = zeros(numImg, 1); % store cont length of all the imgs

for indImg = 1:numImg
    [rowSet(indImg), lenSet(indImg)] = size(imgContSet{indImg});
end

len = max(lenSet); % max cont length of all the conts in the training imgs
row = sum(rowSet);

% all the conts are been put into dcode ***
dcode = zeros(row, len);
tempIndRow = 1;
for indImg = 1:numImg
    dcode(tempIndRow:tempIndRow+rowSet(indImg)-1, 1:lenSet(indImg)) = ...
        imgContSet{indImg}(1:rowSet(indImg), 1:lenSet(indImg));
    tempIndRow = tempIndRow + rowSet(indImg);
end

% step 3: transfer the n,e,s,w symbols to l,s,r symbols ***
ccode = dcode;
for i = 1:row
    [~, ind] = min(dcode(i,:));
    dcode1 = dcode(i,1:ind-1);
    
    for j = 1:ind - 1
        if j == 1,
            ccode(i,j) = dcode1(1);  % fist chain is in absolute term
        elseif j <= ind - 1,
            if dcode1(j) == dcode1(j-1),
                ccode(i,j) = 2;     % straight
            elseif dcode1(j) - dcode1(j-1) == 1,
                ccode(i,j) = 3;     % right
            elseif dcode1(j) ==1 && dcode1(j-1) == 4,
                ccode(i,j) = 3;
            else
                ccode(i,j) = 1;     % left
            end
        end
    end %  for j = 1:ind-1
end % for i=1:row

% step 4: get the maximum depth D of the context tree, D = ceil(log L/Log 3) ***
L = numel(find(ccode(:)));
orderRange = ceil(log(L)/log(3));

countStateSpace = zeros(orderRange, 3^orderRange, 3);

stat2IndVec = zeros(orderRange, orderRange);
for i = 1:orderRange
    stat2IndVec(i, 1:i) = 3.^(0:i-1);
end

% step 5: traverse all the conts once to get the count of all the contexts ***
% step 5.1: check whether to count all the conts ***
if flag == 0 % means to count all the conts
    flag = ones(row, 1);
end

if strcmp(treePrune, 'CT-LL') == 1 % only consider maxCounters
    numCounter = 0;
    
    % step 5.2: traverse all the conts once, get maxCountersRedun conunters ***
    for i = 1:row
        % skip if current contour is not in the class ******
        if flag(i) == 1
            [~, ind] = min(ccode(i,:));
            ccode1 = ccode(i,1:ind-1);         % read the i-th contour
            
            % not count symbols with index <= orderRange to make total num of ***
            % symbols is equal for different order of contexs ***
            for j = 1 + orderRange:ind - 1
                for iorder = 1:orderRange
                    % if j - iorder >= 1 % to make sure enough previous symbols
                    p = zeros(iorder, 1); % store previous states
                    
                    p(1:iorder) = ccode1(j-iorder:j-1);
                    cur = ccode1(j);
                    
                    countIndex = stat2IndVec(iorder, 1:iorder) * (p - 1) + 1;
                    
                    tempCountSum = sum(countStateSpace(iorder, countIndex, :));
                    
                    if tempCountSum == 0
                        numCounter = numCounter + 1;
                    end
                    
                    % restrict the conunter numbers by maxCountersRedun ***
                    if numCounter <= maxCountersRedun || tempCountSum ~= 0
                        if cur == 1 % left
                            countStateSpace(iorder, countIndex, 1) = 1 + ...
                                countStateSpace(iorder, countIndex, 1);
                        elseif cur == 2 % straight
                            countStateSpace(iorder, countIndex, 2) = 1 + ...
                                countStateSpace(iorder, countIndex, 2);
                        else
                            countStateSpace(iorder, countIndex, 3) = 1 + ...
                                countStateSpace(iorder, countIndex, 3);
                        end
                    end
                    %  end % j-iorder > = 1
                end % iorder = 1:orderRange
            end
            
        end % flag(i) == 1
    end % for j = 1:row
    
    % step 5.3: choose maxCounters counters from maxCountersRedun conunters, others are set to zero ***
    countStateSpaceSum = sum(countStateSpace, 3);
    if numCounter <= maxCounters % no need to choose
        minCount = min(find(countStateSpaceSum));
    else
        [tempVal, tempInd] = sort(countStateSpaceSum(:), 'descend');
        minCount = tempVal(maxCounters+1);
        [tempIndX, tempIndY] = ind2sub([orderRange, 3^orderRange], tempInd(maxCounters+1:end));
        for i = 1:length(tempIndX)
            countStateSpace(tempIndX(i), tempIndY(i), :) = 0;
        end
    end
end

end